import os

from hdmf.common import DynamicTable
from hdmf.utils import docval, get_docval, popargs
from pynwb.core import NWBDataInterface
from pynwb.file import MultiContainerInterface
from pynwb.io.core import NWBContainerMapper

from pynwb import get_class, load_namespaces, register_class, register_map

namespace_name = "ndx-acquisition-module"

# Set path of the namespace file to the expected install location
# If the extension has not been installed yet but we are running directly from
# the git repo

namespace_path = os.path.join("spec", namespace_name + ".namespace.yaml")
initfile_path = os.path.dirname(__file__)

namespace_path = os.path.join(initfile_path, namespace_path)
if not os.path.exists(namespace_path):
    namespace_path = os.path.abspath(
        os.path.join(initfile_path, "..", "..", "..", namespace_path)
    )

# Load the namespace
load_namespaces(namespace_path)

# Create API
# very similar to how ProcessingModule is created

AcquisitionModule = get_class("AcquisitionModule", namespace_name)
init_docval = get_docval(AcquisitionModule.__init__, "name", "description")


@register_class("AcquisitionModule", namespace_name)
class AcquisitionModule(MultiContainerInterface):
    __clsconf__ = [
        {
            "attr": "data_interfaces",
            "add": "add",
            "type": (NWBDataInterface, DynamicTable),
            "get": "get",
        }
    ]

    @docval(
        # obtain docval from namespace
        *init_docval,
        {
            "name": "data_interfaces",
            "doc": "NWBDataInterface",
            "type": (list, tuple, dict, NWBDataInterface, DynamicTable),
            "default": None,
        },
    )
    def __init__(self, **kwargs):
        description, data_interfaces = popargs("description", "data_interfaces", kwargs)
        super().__init__(**kwargs)
        self.description = description
        self.data_interfaces = data_interfaces


@register_map(AcquisitionModule)
class ModuleMap(NWBContainerMapper):
    def __init__(self, spec):
        super().__init__(spec)
        containers_spec = self.spec.get_neurodata_type("NWBDataInterface")
        table_spec = self.spec.get_neurodata_type("DynamicTable")
        self.map_spec("data_interfaces", containers_spec)
        self.map_spec("data_interfaces", table_spec)

    @NWBContainerMapper.constructor_arg("name")
    def name(self, builder, manager):
        return builder.name



import os
from setuptools import build_meta as _orig
from shutil import copy2


prepare_metadata_for_build_wheel = _orig.prepare_metadata_for_build_wheel
build_wheel = _orig.build_wheel
build_sdist = _orig.build_sdist
get_requires_for_build_sdist = _orig.get_requires_for_build_sdist


def _copy_spec_files(project_dir):
    ns_path = os.path.join(project_dir, 'spec', 'ndx-acquisition-module.namespace.yaml')
    ext_path = os.path.join(project_dir, 'spec', 'ndx-acquisition-module.extensions.yaml')

    dst_dir = os.path.join(project_dir, 'src', 'pynwb', 'ndx_acquisition_module', 'spec')

    if not os.path.exists(dst_dir):
        os.mkdir(dst_dir)

    copy2(ns_path, dst_dir)
    copy2(ext_path, dst_dir)


def get_requires_for_build_wheel(config_settings=None):
    # hijacking this function to copy necessary spec files
    _copy_spec_files(os.path.join(os.path.dirname(__file__), '..'))
    return _orig.get_requires_for_build_wheel(config_settings)
